"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("assert");
const uuid = require("uuid");
const queue_1 = require("./queue");
const log4js = require("@log4js-node/log4js-api");
const logger = log4js.getLogger('rabbit-queue');
let replyHandlers = {};
function createReplyQueue(channel) {
    return __awaiter(this, void 0, void 0, function* () {
        yield channel.assertQueue('', { exclusive: true }).then(replyTo => {
            channel.replyName = replyTo.queue;
            channel.consume(channel.replyName, onReply, { noAck: true });
        });
    });
}
exports.createReplyQueue = createReplyQueue;
function addHandler(correlationId, handler) {
    assert(!replyHandlers[correlationId], `Already added reply handler with this id: ${correlationId}.`);
    replyHandlers[correlationId] = handler;
}
exports.addHandler = addHandler;
function getReply(content, headers = {}, channel, cb) {
    return new Promise((resolve, reject) => {
        var msg = JSON.stringify(content);
        var correlationId = headers.correlationId || uuid.v4();
        headers = Object.assign({
            persistent: false,
            correlationId,
            replyTo: channel.replyName
        }, headers);
        const bufferContent = Buffer.from(msg);
        addHandler(correlationId, (err, body) => (err ? reject(err) : resolve(body)));
        cb(bufferContent, headers, correlationId, (err, ok) => (err ? reject(err) : {}));
    });
}
exports.getReply = getReply;
function onReply(msg) {
    const id = msg.properties.correlationId;
    const replyHandler = replyHandlers[id];
    if (!replyHandler) {
        logger.error(`No reply Handler found for ${id}`);
        return;
    }
    delete replyHandlers[id];
    const body = msg.content.toString();
    logger.info(`[${id}] <- Returning reply ${msg.content.byteLength} bytes`);
    const obj = JSON.parse(body);
    if (obj && obj.error && obj.error_code === queue_1.default.ERROR_DURING_REPLY.error_code) {
        replyHandler(new Error(obj.error_message), null);
    }
    else {
        replyHandler(null, obj);
    }
}
//# sourceMappingURL=replyQueue.js.map